#!/bin/bash

################################################################################
#@(#)ebook_compress ver1.0.0 2020.07.07
#
#  概要:
#	PDFファイルを解像度を落としPDFファイルサイズを縮小します。
#	変換前と変換後のディレクトリを比較し、未変換のファイルのみ変換処理します。
#	存在しないディレクトリは、変換前のディレクトリ構造を維持します。
#
#  使い方:
#	本スクリプトを実行すれば、指定した変換前ディレクトリと変換後ディレクトリを比較し、処理が始まります。
#
#  動作環境：
#	OS：Windows10
#	ツール：WSL
#	ディストリビューション：Ubuntu 16.04.2 LTS
#	注意事項：WSL以外のUbuntuでも動作可能。CentOSではrenameコマンドの書式が異なる。
#
#
#  事前準備：下記のパッケージを導入する。
#	パッケージ：ghostscript
#	必要な理由：PDFの圧縮処理に必要		
#	導入方法：		
#	sudo apt update
#	sudo apt install ghostscript
#
#
#	パッケージ：rename
#	必要な理由：ファイルのリネーム処理に利用
#	導入方法：		
#	sudo apt-get install rename
#
#  必須作業：
#	下記の定数定義の値は、環境により変更必須。
#	BEFORE_DIR：圧縮対象のPDFが存在するフォルダ
#	AFTER_DIR：圧縮後のPDFを出力するフォルダ
#	※AFTER_DIRのフォルダ構造・ファイル名は、BEFORE_DIRの内容を維持します。
#
#  変更履歴:
#	2020/07/07 1.0.0  新規作成。
#
################################################################################

################################################################################
## 変数・定数定義
################################################################################
##変数定義
#trap定義 mktempで作成した一時ファイルをスクリプト終了時に自動削除
trap "rm -f /tmp/tmp.*; exit 1" 0 1 3 15

#スクリプトで利用する一時ファイルを作成。/tmp/tmp.<ランダム名>.--suffix名のファイルを作成
tmp_dest_file_name=`mktemp --suffix=-dest`        #compress元ファイル一覧
tmp_src_file_name=`mktemp --suffix=-src`          #compress先ファイル一覧
tmp_diff_file_name=`mktemp --suffix=-diff`        #未compressファイル名一覧
tmp_convert_list_tmp=`mktemp --suffix=-list_tmp`  #未compressファイル名フルパス（未sort）
tmp_convert_list=`mktemp --suffix=-list`          #未compressファイル名フルパス（sort済）
tmp_src_move_list=`mktemp --suffix=-src_move`     #compress済みファイル移動元リスト
tmp_dest_move_list=`mktemp --suffix=-dest_move`   #compress済みファイル移動先リスト

##定数定義
#compress元ディレクトリ
readonly BEFORE_DIR='/mnt/g/ebook_origin'

#compress先ディレクトリ
readonly AFTER_DIR='/mnt/g/ebook_compress'

#compress作業ディレクトリ
readonly TMP_DIR="$AFTER_DIR/tmp"

#compressログ
readonly LOG_DIR="$AFTER_DIR/log"


#区切り文字を改行コードへ変換###################################################
#スペースなどを区切り文字と認識しないように対策
IFS=$'\n'


################################################################################
## ディレクトリ作成
################################################################################

#ログディレクトリ作成###########################################################
[ ! -e $LOG_DIR ] && mkdir -p $LOG_DIR

#作業ディレクトリ作成###########################################################
[ ! -e $TMP_DIR ] && mkdir -p $TMP_DIR

#compress先ディレクトリ作成#####################################################
#ディレクトリ構造取得、置換
#sedはファイルパスを扱うため、/ではなく#で指定
array_convert_dir=`find $BEFORE_DIR -type d | sed -e s#$BEFORE_DIR#$AFTER_DIR#g`

#compress先ディレクトリ作成
for dir in $array_convert_dir;
do
  mkdir -p $dir
done



################################################################################
## ファイルリスト作成
################################################################################

##compress対象ファイルリスト作成################################################
#compress元ファイル一覧取得
find $BEFORE_DIR -type f -name *.pdf | gawk -F/ '{print $NF}' | sort --version-sort > $tmp_src_file_name

#compress先ファイル一覧取得
find $AFTER_DIR -type f -name *.pdf | gawk -F/ '{print $NF}' | sort --version-sort > $tmp_dest_file_name

#未compressファイル名一覧作成
###diffオプション説明###
# --side-by-side:2列出力
# --suppress-common-lines:side-by-side形式で共通な行を表示しない
###
diff --side-by-side --suppress-common-lines $tmp_src_file_name $tmp_dest_file_name | gawk '{print $1}' > $tmp_diff_file_name

#未compressファイル　フルパスリスト作成#########################################
cat $tmp_diff_file_name | while read line
do
  find $BEFORE_DIR -type f -name *${line}* >> $tmp_convert_list_tmp
done

#ファイル名ソート
###sortオプション説明###
# --version-sort:自然な（バージョン）数字順で並べ替える
###
cat $tmp_convert_list_tmp | sort --version-sort > $tmp_convert_list


################################################################################
## compress処理
################################################################################
##pdf圧縮処理→圧縮後ファイルを一時フォルダに移動→ログファイルをログフォルダへ移動

###xargオプション説明###
# -t:実行前にコマンドラインを標準エラー出力へ出力する
# --no-run-if-empty:標準入力が空の場合は指定したコマンドを実行しない
# -L1:1コマンドラインで1行を利用する指定
# -P6:同時に実行するプロセス数の上限を指定する
# -I:xargs実行時に指定したコマンドの引数のうち、置換文字部分を標準入力から読み込んだ名前で置き換える
# 注意:ファイル名に空白、記号が入っても処理できるように{}は'{}'とする
###

###gs(Ghostscript)オプション説明###
# -dNOPAUSE:入力ファイルを1つ処理完了したら、次の入力ファイルを処理する
# -dBATCH:変換処理が完了したらgsを終了する
# -sDEVICE=pdfwrite:ファイルの出力方式を指定。PDFファイルに変換を指定している。
# -dCompatibilityLevel=1.4:PDFバージョンを指定。ここでは1.4を指定している。
# -dPDFSETTINGS=/ebook:出力用途(品質)を指定。ここではebook(電子図書)を指定している。
# -sOutputFile:変換後の出力先パス、ファイル名を指定。
###

cat $tmp_convert_list | \
  xargs -t --no-run-if-empty -L1 -P6 -I{} bash -c \
  "\
  gs -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -dCompatibilityLevel=1.4 -dPDFSETTINGS=/ebook -sOutputFile='{}'.ebook '{}' &>> '{}'.log;\
  mv '{}'.ebook $TMP_DIR;\
  mv '{}'.log $LOG_DIR
  "

################################################################################
## 圧縮後ファイル最終処理
################################################################################
#ファイルリネーム###############################################################
#一時フォルダに移動したファイルを変更
rename "s/.pdf.ebook/.pdf/" $TMP_DIR/*

#ファイル移動処理###############################################################
#移動元ファイルリスト作成
###lsオプション説明###
# -1:ファイル名を1列で表示。
###
ls -1 $TMP_DIR > $tmp_src_move_list

#移動先ファイルパスリスト作成
cat $tmp_convert_list | sed -e s#$BEFORE_DIR#$AFTER_DIR#g > $tmp_dest_move_list

#圧縮後ファイルを一時フォルダから本来あるべきフォルダへファイル移動
cat $tmp_src_move_list | while read line
do
  cat $tmp_dest_move_list | grep $line | sed -z 's/$line//' | xargs -I{} mv $TMP_DIR/$line {}
done



################################################################################
## 圧縮後差分ファイルリスト作成
################################################################################
##compress対象ファイルリスト作成################################################
#compress元ファイル一覧取得
find $BEFORE_DIR -type f -name *.pdf | gawk -F/ '{print $NF}' | sort --version-sort > $tmp_src_file_name

#compress先ファイル一覧取得
find $AFTER_DIR -type f -name *.pdf | gawk -F/ '{print $NF}' | sort --version-sort > $tmp_dest_file_name

#差分ファイルリスト作成
###diffオプション説明###
# --side-by-side:2列出力
# --suppress-common-lines:side-by-side形式で共通な行を表示しない
###
diff --side-by-side --suppress-common-lines $tmp_src_file_name $tmp_dest_file_name | gawk '{print $1}' > $LOG_DIR/`date +"%Y%m%d-%H%M%S"`.diff



exit 0

